@@ -50,7 +50,6 @@ class ConfigManager(object):
             'tip.autostart': False,
             'tip.position': 0,
             'form.toolbar': True,
-            'form.statusbar': True,
             'client.default_width': 900,
             'client.default_height': 750,
             'client.modepda': False,
@@ -58,7 +57,7 @@ class ConfigManager(object):
             'client.form_tab': form_tab,
             'client.maximize': False,
             'client.save_width_height': True,
-            'client.save_tree_expanded_state': True,
+            'client.save_tree_state': True,
             'client.spellcheck': False,
             'client.default_path': get_home_dir(),
             'client.lang': locale.getdefaultlocale()[0],
