@@ -5,12 +5,13 @@ from interface import ParserView
 
 class ViewGraph(ParserView):
 
-    def __init__(self, screen, widget, children=None, buttons=None,
+    def __init__(self, screen, widget, children=None, state_widgets=None,
             notebooks=None, cursor_widget=None, children_field=None):
-        super(ViewGraph, self).__init__(screen, widget, children, buttons,
-            notebooks, cursor_widget, children_field)
+        super(ViewGraph, self).__init__(screen, widget, children,
+            state_widgets, notebooks, cursor_widget, children_field)
         self.view_type = 'graph'
         self.widgets = children
+        self.editable = False
 
     def __getitem__(self, name):
         return None
@@ -20,9 +21,9 @@ class ViewGraph(ParserView):
         for widget in self.widgets.keys():
             self.widgets[widget].destroy()
             del self.widgets[widget]
-        del self.widget
-        del self.screen
-        del self.buttons
+        self.widget = None
+        self.screen = None
+        self.state_widgets = None
 
     def cancel(self):
         pass
